@@ -114,8 +114,66 @@ class CalDAVRequest
     $this->options = $options;
     if ( !isset($this->options['allow_by_email']) ) $this->options['allow_by_email'] = false;
 
+    /**
+    * Our path is /<script name>/<user name>/<user controlled> if it ends in
+    * a trailing '/' then it is referring to a DAV 'collection' but otherwise
+    * it is referring to a DAV data item.
+    *
+    * Permissions are controlled as follows:
+    *  1. if there is no <user name> component, the request has read privileges
+    *  2. if the requester is an admin, the request has read/write priviliges
+    *  3. if there is a <user name> component which matches the logged on user
+    *     then the request has read/write privileges
+    *  4. otherwise we query the defined relationships between users and use
+    *     the minimum privileges returned from that analysis.
+    */
+    $this->path = (isset($_SERVER['PATH_INFO']) ? $_SERVER['PATH_INFO'] : "/");
+    $this->path = rawurldecode($this->path);
+
+    /** Allow a request for .../calendar.ics to translate into the calendar URL */
+    if ( preg_match( '#^(/[^/]+/[^/]+).ics$#', $this->path, $matches ) ) {
+      $this->path = $matches[1]. '/';
+    }
+
+    // dbg_error_log( "caldav", "Sanitising path '%s'", $this->path );
+    $bad_chars_regex = '/[\\^\\[\\(\\\\]/';
+    if ( preg_match( $bad_chars_regex, $this->path ) ) {
+      $this->DoResponse( 400, translate("The calendar path contains illegal characters.") );
+    }
+    if ( strstr($this->path,'//') ) $this->path = preg_replace( '#//+#', '/', $this->path);
+
     if ( !isset($c->raw_post) ) $c->raw_post = file_get_contents( 'php://input');
-    $this->raw_post = $c->raw_post;
+    if ( isset($_SERVER['HTTP_CONTENT_ENCODING']) ) {
+      $encoding = $_SERVER['HTTP_CONTENT_ENCODING'];
+      @dbg_error_log('caldav', 'Content-Encoding: %s', $encoding );
+      $encoding = preg_replace('{[^a-z0-9-]}i','',$encoding);
+      if ( ! ini_get('open_basedir') && (isset($c->dbg['ALL']) || isset($c->dbg['caldav'])) ) {
+        $fh = fopen('/tmp/encoded_data.'.$encoding,'w');
+        if ( $fh ) {
+          fwrite($fh,$c->raw_post);
+          fclose($fh);
+        }
+      }
+      switch( $encoding ) {
+        case 'gzip':
+          $this->raw_post = @gzdecode($c->raw_post);
+          break;
+        case 'deflate':
+          $this->raw_post = @gzinflate($c->raw_post);
+          break;
+        case 'compress':
+          $this->raw_post = @gzuncompress($c->raw_post);
+          break;
+        default:
+      }
+      if ( empty($this->raw_post) && !empty($c->raw_post) ) {
+        $this->PreconditionFailed(415, 'content-encoding', sprintf('Unable to decode "%s" content encoding.', $_SERVER['HTTP_CONTENT_ENCODING']));
+      }
+      $c->raw_post = $this->raw_post;
+    }
+    else {
+      $this->raw_post = $c->raw_post;
+    }
 
     if ( isset($debugging) && isset($_GET['method']) ) {
       $_SERVER['REQUEST_METHOD'] = $_GET['method'];
@@ -136,6 +194,9 @@ class CalDAVRequest
           $this->content_type = 'text/xml';
         }
       }
+      else if ( $this->method == 'PUT' || $this->method == 'POST' ) {
+        $this->CoerceContentType();
+      }
     }
     $this->user_agent = ((isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : "Probably Mulberry"));
 
@@ -215,34 +276,6 @@ class CalDAVRequest
       if ( ! isset($this->timeout) || $this->timeout == 0 ) $this->timeout = (isset($c->default_lock_timeout) ? $c->default_lock_timeout : 900);
     }
 
-    /**
-    * Our path is /<script name>/<user name>/<user controlled> if it ends in
-    * a trailing '/' then it is referring to a DAV 'collection' but otherwise
-    * it is referring to a DAV data item.
-    *
-    * Permissions are controlled as follows:
-    *  1. if there is no <user name> component, the request has read privileges
-    *  2. if the requester is an admin, the request has read/write priviliges
-    *  3. if there is a <user name> component which matches the logged on user
-    *     then the request has read/write privileges
-    *  4. otherwise we query the defined relationships between users and use
-    *     the minimum privileges returned from that analysis.
-    */
-    $this->path = (isset($_SERVER['PATH_INFO']) ? $_SERVER['PATH_INFO'] : "/");
-    $this->path = rawurldecode($this->path);
-
-    /** Allow a request for .../calendar.ics to translate into the calendar URL */
-    if ( preg_match( '#^(/[^/]+/[^/]+).ics$#', $this->path, $matches ) ) {
-      $this->path = $matches[1]. '/';
-    }
-
-    // dbg_error_log( "caldav", "Sanitising path '%s'", $this->path );
-    $bad_chars_regex = '/[\\^\\[\\(\\\\]/';
-    if ( preg_match( $bad_chars_regex, $this->path ) ) {
-      $this->DoResponse( 400, translate("The calendar path contains illegal characters.") );
-    }
-    if ( strstr($this->path,'//') ) $this->path = preg_replace( '#//+#', '/', $this->path);
-
     $this->principal = new Principal('path',$this->path);
 
     /**
@@ -289,9 +322,9 @@ class CalDAVRequest
       $params[':resourcetypes'] = sprintf('<DAV::collection/><urn:ietf:params:xml:ns:caldav:%s/>', $this->collection_type );
       $sql = <<<EOSQL
 INSERT INTO collection ( user_no, parent_container, dav_name, dav_displayname, is_calendar, created, modified, dav_etag, resourcetypes )
-    VALUES( (SELECT user_no FROM usr WHERE username = :username),
+    VALUES( (SELECT user_no FROM usr WHERE username = text(:username)),
             :parent_container, :dav_name,
-            (SELECT fullname FROM usr WHERE username = :username) || :boxname,
+            (SELECT fullname FROM usr WHERE username = text(:username)) || :boxname,
              FALSE, current_timestamp, current_timestamp, '1', :resourcetypes )
 EOSQL;
 
@@ -814,7 +847,11 @@ EOSQL;
     if ( isset($this->content_type) ) {
       $type = explode( '/', $this->content_type, 2);
       /** @todo: Perhaps we should look at the target collection type, also. */
-      if ( $type[0] == 'text' ) return;
+      if ( $type[0] == 'text' ) {
+        if ( !empty($type[1]) && ($type[1] == 'vcard' || $type[1] == 'calendar' || $type[1] == 'x-vcard') ) {
+          return;
+        }
+      }
     }
 
     /** Null (or peculiar) content-type supplied so we have to try and work it out... */
@@ -840,7 +877,7 @@ EOSQL;
         dbg_error_log( 'LOG NOTICE', 'Unusual content-type of "%s" and first word of content is "%s"',
                                         (isset($this->content_type)?$this->content_type:'(null)'), $first_word );
     }
-    $this->content_type = 'text/plain';
+    if ( empty($this->content_type) ) $this->content_type = 'text/plain';
   }
 
 
@@ -1086,11 +1123,11 @@ EOSQL;
   * @param string $precondition The namespaced precondition tag.
   * @param string $explanation An optional text explanation for the failure.
   */
-  function PreconditionFailed( $status, $precondition, $explanation = '') {
+  function PreconditionFailed( $status, $precondition, $explanation = '', $xmlns='DAV:') {
     $xmldoc = sprintf('<?xml version="1.0" encoding="utf-8" ?>
-<error xmlns="DAV:">
+<error xmlns="%s">
   <%s/>%s
-</error>', str_replace('DAV::', '', $precondition), $explanation );
+</error>', $xmlns, str_replace($xmlns.':', '', $precondition), $explanation );
 
     $this->DoResponse( $status, $xmldoc, 'text/xml; charset="utf-8"' );
     exit(0);  // Unecessary, but might clarify things
