@@ -360,9 +360,9 @@ class DAVResource
       $params[':resourcetypes'] = sprintf('<DAV::collection/><urn:ietf:params:xml:ns:caldav:%s/>', $this->collection_type );
       $sql = <<<EOSQL
 INSERT INTO collection ( user_no, parent_container, dav_name, dav_displayname, is_calendar, created, modified, dav_etag, resourcetypes )
-    VALUES( (SELECT user_no FROM usr WHERE username = :username),
+    VALUES( (SELECT user_no FROM usr WHERE username = text(:username)),
             :parent_container, :dav_name,
-            (SELECT fullname FROM usr WHERE username = :username) || :boxname,
+            (SELECT fullname FROM usr WHERE username = text(:username)) || :boxname,
              FALSE, current_timestamp, current_timestamp, '1', :resourcetypes )
 EOSQL;
       $qry = new AwlQuery( $sql, $params );
@@ -1590,8 +1590,20 @@ EOQRY;
 
       case 'urn:ietf:params:xml:ns:caldav:supported-calendar-component-set':
         if ( ! $this->_is_collection ) return false;
-        if ( $this->IsCalendar() )
-          $set_of_components = array( 'VEVENT', 'VTODO', 'VJOURNAL', 'VTIMEZONE', 'VFREEBUSY' );
+        if ( $this->IsCalendar() ) {
+          if ( !isset($this->dead_properties) ) $this->FetchDeadProperties();
+          if ( isset($this->dead_properties[$tag]) ) {
+            $set_of_components = explode('"', $this->dead_properties[$tag]);
+            foreach( $set_of_components AS $k => $v ) {
+              if ( !preg_match('{(VEVENT|VTODO|VJOURNAL|VTIMEZONE|VFREEBUSY)}', $v) ) {
+                unset( $set_of_components[$k] );
+              }
+            }
+          }
+          else {
+            $set_of_components = array( 'VEVENT', 'VTODO', 'VJOURNAL', 'VTIMEZONE', 'VFREEBUSY' );
+          }
+        }
         else if ( $this->IsSchedulingCollection() )
           $set_of_components = array( 'VEVENT', 'VTODO', 'VFREEBUSY' );
         else return false;
