@@ -242,7 +242,8 @@ function handle_schedule_request( $ical ) {
     $content = $ncal->Render();
     $cid = $ar->GetProperty('collection_id');
     dbg_error_log('DELIVER', 'to user: %s, to path: %s, collection: %s, from user: %s, caldata %s', $attendee_principal->user_no(), $deliver_path, $cid, $request->user_no, $content );
-    write_resource( new DAVResource($deliver_path . $etag . '.ics'), $content, $ar, $request->user_no, md5($content),
+    $item_etag = md5($content);
+    write_resource( new DAVResource($deliver_path . $etag . '.ics'), $content, $ar, $request->user_no, $item_etag,
                     $put_action_type='INSERT', $caldav_context=true, $log_action=true, $etag );
     $attendee->SetParameterValue ('SCHEDULE-STATUS','1.2;Scheduling message has been delivered');
   }
@@ -252,7 +253,8 @@ function handle_schedule_request( $ical ) {
   $content = $ncal->Render();
   $deliver_path = $request->principal->internal_url('schedule-inbox');
   $ar = new DAVResource($deliver_path);
-  write_resource( new DAVResource($deliver_path . $etag . '.ics'), $content, $ar, $request->user_no, md5($content),
+  $item_etag = md5($content);
+  write_resource( new DAVResource($deliver_path . $etag . '.ics'), $content, $ar, $request->user_no, $item_etag,
                      $put_action_type='INSERT', $caldav_context=true, $log_action=true, $etag );
   //$etag = md5($content);
   header('ETag: "'. $etag . '"' );
@@ -310,73 +312,240 @@ function handle_schedule_reply ( vCalendar $ical ) {
 
 
 /**
-* Create/Update the scheduling requests for this resource.  This includes updating
-* the scheduled user's default calendar.
-* @param vComponent $resource The VEVENT/VTODO/... resource we are scheduling
-* @param boolean $create true if the scheduling requests are being created.
-*/
-function do_scheduling_requests( vCalendar $resource, $create ) {
-  global $request, $c;
-  if ( isset($c->enable_auto_schedule) && !$c->enable_auto_schedule ) return;
-  
-  if ( ! is_object($resource) ) {
-    dbg_error_log( 'PUT', 'do_scheduling_requests called with non-object parameter (%s)', gettype($resource) );
-    return;
+ * Do the scheduling adjustments for a REPLY when an ATTENDEE updates their status.
+ * @param vCalendar $resource The resource that the ATTENDEE is writing to their calendar
+ * @param string $organizer The property which is the event ORGANIZER.
+ */
+function do_scheduling_reply( vCalendar $resource, vProperty $organizer ) {
+  global $request;
+  $organizer_email = preg_replace( '/^mailto:/i', '', $organizer->Value() );
+  $organizer_principal = new Principal('email',$organizer_email );
+  if ( !$organizer_principal->Exists() ) {
+    dbg_error_log( 'PUT', 'Organizer "%s" not found - cannot perform scheduling reply.', $organizer );
+    return false;
   }
-
+  $sql = 'SELECT caldav_data.dav_name, caldav_data.caldav_data FROM caldav_data JOIN calendar_item USING(dav_id) ';
+  $sql .= 'WHERE caldav_data.collection_id IN (SELECT collection_id FROM collection WHERE is_calendar AND user_no =?) ';
+  $sql .= 'AND uid=? LIMIT 1';
+  $uids = $resource->GetPropertiesByPath('/VCALENDAR/*/UID');
+  if ( count($uids) == 0 ) {
+    dbg_error_log( 'PUT', 'No UID in VCALENDAR - giving up on REPLY.' );
+    return false;
+  }
+  $uid = $uids[0]->Value();
+  $qry = new AwlQuery($sql,$organizer_principal->user_no(), $uid);
+  if ( !$qry->Exec('PUT',__LINE__,__FILE__) || $qry->rows() < 1 ) {
+    dbg_error_log( 'PUT', 'Could not find original event from organizer - giving up on REPLY.' );
+    return false;
+  }
+  $row = $qry->Fetch();
   $attendees = $resource->GetAttendees();
-  if ( count($attendees) == 0 ) {
-    dbg_error_log( 'PUT', 'Event has no attendees - no scheduling required.', count($attendees) );
-    return;
+  foreach( $attendees AS $v ) {
+    $email = preg_replace( '/^mailto:/i', '', $v->Value() );
+    if ( $email == $request->principal->email() ) {
+      $attendee = $v;
+    }
+  }
+  if ( empty($attendee) ) {
+    dbg_error_log( 'PUT', 'Could not find ATTENDEE in VEVENT - giving up on REPLY.' );
+    return false;
+  }
+  $schedule_original = new vCalendar($row->caldav_data);
+  $schedule_original->UpdateAttendeeStatus($request->principal->email(), clone($attendee) );
+
+  $collection_path = preg_replace('{/[^/]+$}', '/', $row->dav_name );
+  $segment_name = str_replace($collection_path, '', $row->dav_name );
+  $organizer_calendar = new WritableCollection(array('path' => $collection_path));
+  $organizer_inbox = new WritableCollection(array('path' => $organizer_principal->internal_url('schedule-inbox')));
+
+  $schedule_reply = clone($schedule_original);
+  $schedule_reply->AddProperty('METHOD', 'REPLY');
+
+  dbg_error_log( 'PUT', 'Writing scheduling REPLY from %s to %s', $request->principal->email(), $organizer_principal->email() );
+  
+  $response = '3.7'; // Organizer was not found on server.
+  if ( !$organizer_calendar->Exists() ) {
+    dbg_error_log('ERROR','Default calendar at "%s" does not exist for user "%s"',
+                              $organizer_calendar->dav_name(), $schedule_target->username());
+    $response = '5.2'; // No scheduling support for user
+  }
+  else {
+    if ( ! $organizer_inbox->HavePrivilegeTo('schedule-deliver-reply') ) {
+      $response = '3.8'; // No authority to deliver replies to organizer.
+    }
+    else if ( $organizer_inbox->WriteCalendarMember($schedule_reply, false, false, $request->principal->username().$segment_name) !== false ) {
+      $response = '1.2'; // Scheduling reply delivered successfully
+      if ( $organizer_calendar->WriteCalendarMember($schedule_original, false, false, $segment_name) === false ) {
+        dbg_error_log('ERROR','Could not write updated calendar member to %s',
+                $attendee_calendar->dav_name(), $attendee_calendar->dav_name(), $schedule_target->username());
+        trace_bug('Failed to write scheduling resource.');
+      }
+    }
   }
 
-  dbg_error_log( 'PUT', 'Adding to scheduling inbox %d attendees', count($attendees) );
-  $schedule_request = clone($resource);
-  $schedule_request->AddProperty('METHOD','REQUEST');
+  $schedule_request = clone($schedule_original);
+  $schedule_request->AddProperty('METHOD', 'REQUEST');
+
+  dbg_error_log( 'PUT', 'Status for organizer <%s> set to "%s"', $organizer->Value(), $response );
+  $organizer->SetParameterValue( 'SCHEDULE-STATUS', $response );
+  $resource->UpdateOrganizerStatus($organizer);
+  $scheduling_actions = true;
+  
+  $attendees = $schedule_original->GetAttendees();
   foreach( $attendees AS $attendee ) {
     $email = preg_replace( '/^mailto:/i', '', $attendee->Value() );
-    if ( $email == $request->principal->email() ) {
-      dbg_error_log( "PUT", "not delivering to owner" );
-      continue;
-    }
+    if ( $email == $request->principal->email() || $email == $organizer_principal->email() ) continue;
+
     $agent = $attendee->GetParameterValue('SCHEDULE-AGENT');
     if ( $agent && $agent != 'SERVER' ) {
       dbg_error_log( "PUT", "not delivering to %s, schedule agent set to value other than server", $email );
       continue;
     }
     $schedule_target = new Principal('email',$email);
+    $response = '3.7'; // Attendee was not found on server.
+    if ( $schedule_target->Exists() ) {
+      $attendee_calendar = new WritableCollection(array('path' => $schedule_target->internal_url('schedule-default-calendar')));
+      if ( !$attendee_calendar->Exists() ) {
+        dbg_error_log('ERROR','Default calendar at "%s" does not exist for user "%s"',
+        $attendee_calendar->dav_name(), $schedule_target->username());
+        $response = '5.2'; // No scheduling support for user
+      }
+      else {
+        $attendee_inbox = new WritableCollection(array('path' => $schedule_target->internal_url('schedule-inbox')));
+        if ( ! $attendee_inbox->HavePrivilegeTo('schedule-deliver-invite') ) {
+          $response = '3.8'; //  No authority to deliver invitations to user.
+        }
+        else if ( $attendee_inbox->WriteCalendarMember($schedule_request, false) !== false ) {
+          $response = '1.2'; // Scheduling invitation delivered successfully
+          if ( $attendee_calendar->WriteCalendarMember($schedule_original, false) === false ) {
+            dbg_error_log('ERROR','Could not write updated calendar member to %s', 
+                    $attendee_calendar->dav_name(), $attendee_calendar->dav_name(), $schedule_target->username());
+            trace_bug('Failed to write scheduling resource.');
+          }
+        }
+      }
+    }
+    dbg_error_log( 'PUT', 'Status for attendee <%s> set to "%s"', $attendee->Value(), $response );
+    $attendee->SetParameterValue( 'SCHEDULE-STATUS', $response );
+    $scheduling_actions = true;
+    
+    $resource->UpdateAttendeeStatus($email, clone($attendee));
+    
+  }
+  
+  return $scheduling_actions;
+}
+
+
+/**
+* Create/Update the scheduling requests for this resource.  This includes updating
+* the scheduled user's default calendar.
+* @param vComponent $resource The VEVENT/VTODO/... resource we are scheduling
+* @param boolean $create true if the scheduling requests are being created.
+* @return true If there was any scheduling action
+*/
+function do_scheduling_requests( vCalendar $resource, $create, $old_data = null ) {
+  global $request, $c;
+  if ( !isset($request) || (isset($c->enable_auto_schedule) && !$c->enable_auto_schedule) ) return false;
+  
+  if ( ! is_object($resource) ) {
+    trace_bug( 'do_scheduling_requests called with non-object parameter (%s)', gettype($resource) );
+    return  false;
+  }
+
+  $organizer = $resource->GetOrganizer();
+  if ( $organizer === false || empty($organizer) ) {
+    dbg_error_log( 'PUT', 'Event has no organizer - no scheduling required.' );
+    return false;
+  }
+  $organizer_email = preg_replace( '/^mailto:/i', '', $organizer->Value() );
+
+  if ( $request->principal->email() != $organizer_email ) {
+    return do_scheduling_reply($resource,$organizer);
+  }
+  
+  $schedule_request = clone($resource);
+  $schedule_request->AddProperty('METHOD', 'REQUEST');
+
+  $old_attendees = array();
+  if ( !empty($old_data) ) {
+    $old_resource = new vCalendar($old_data);
+    $old_attendees = $old_resource->GetAttendees();
+  }
+  $attendees = $resource->GetAttendees();
+  if ( count($attendees) == 0 && count($old_attendees) == 0 ) {
+    dbg_error_log( 'PUT', 'Event has no attendees - no scheduling required.', count($attendees) );
+    return false;
+  }
+  $removed_attendees = array();
+  foreach( $old_attendees AS $attendee ) {
+    $email = preg_replace( '/^mailto:/i', '', $attendee->Value() );
+    if ( $email == $request->principal->email() ) continue;
+    $removed_attendees[$email] = $attendee;
+  }
+
+  dbg_error_log( 'PUT', 'Writing scheduling resources for %d attendees', count($attendees) );
+  $scheduling_actions = false;
+  foreach( $attendees AS $attendee ) {
+    $email = preg_replace( '/^mailto:/i', '', $attendee->Value() );
+    if ( $email == $request->principal->email() ) {
+      dbg_error_log( "PUT", "not delivering to owner '%s'", $request->principal->email() );
+      continue;
+    }
+
+    if ( $create ) {
+      $attendee_is_new = true;
+    }
+    else {
+      $attendee_is_new = !isset($removed_attendees[$email]);
+      if ( !$attendee_is_new ) unset($removed_attendees[$email]);
+    }
+
+    $agent = $attendee->GetParameterValue('SCHEDULE-AGENT');
+    if ( $agent && $agent != 'SERVER' ) {
+      dbg_error_log( "PUT", "not delivering to %s, schedule agent set to value other than server", $email );
+      continue;
+    }
+    $schedule_target = new Principal('email',$email);
+    $response = '3.7';  // Attendee was not found on server.
+    dbg_error_log( 'PUT', 'Handling scheduling resources for %s on %s which is %s', $email,
+                     ($create?'create':'update'), ($attendee_is_new? 'new' : 'an update') );
     if ( $schedule_target->Exists() ) {
       $attendee_calendar = new WritableCollection(array('path' => $schedule_target->internal_url('schedule-default-calendar')));
       if ( !$attendee_calendar->Exists() ) {
         dbg_error_log('ERROR','Default calendar at "%s" does not exist for user "%s"',
                       $attendee_calendar->dav_name(), $schedule_target->username());
-        $response = '5.3;'.translate('No scheduling support for user');
-      }
-      else if ( $attendee_calendar->WriteCalendarMember($resource, $create) === false ) {
-        dbg_error_log('ERROR','Could not write new calendar member to %s', $attendee_calendar->dav_name(),
-        $attendee_calendar->dav_name(), $schedule_target->username());
-        $response = '5.3;'.translate('No scheduling support for user');
+        $response = '5.2';  // No scheduling support for user
       }
       else {
         $attendee_inbox = new WritableCollection(array('path' => $schedule_target->internal_url('schedule-inbox')));
         if ( ! $attendee_inbox->HavePrivilegeTo('schedule-deliver-invite') ) {
-          $response = '3.8;'.translate('No authority to deliver invitations to user.');
+          $response = '3.8';  // No authority to deliver invitations to user.
         }
-        else if ( $attendee_inbox->WriteCalendarMember($schedule_request, $create) === false ) {
-          $response = '5.3;'.translate('No scheduling support for user');
-        }
-        else {
-          $response = '2.0;'.translate('Scheduling invitation delivered successfully');
+        else if ( $attendee_inbox->WriteCalendarMember($schedule_request, $attendee_is_new) !== false ) {
+          $response = '1.2';  // Scheduling invitation delivered successfully
+          if ( $attendee_calendar->WriteCalendarMember($resource, $attendee_is_new) === false ) {
+                dbg_error_log('ERROR','Could not write %s calendar member to %s', ($attendee_is_new?'new':'updated'),
+                        $attendee_calendar->dav_name(), $attendee_calendar->dav_name(), $schedule_target->username());
+                trace_bug('Failed to write scheduling resource.');
+          }
         }
       }
     }
-    else {
-      $response = '5.3;'.translate('No scheduling support for user');
-    }
-    $schedule_status = '"'.$response.'"';
-    dbg_error_log( 'PUT', 'Status for attendee <%s> set to "%s"', $attendee->Value(), $schedule_status );
-    $attendee->SetParameterValue( 'SCHEDULE-STATUS', $schedule_status );
+    dbg_error_log( 'PUT', 'Status for attendee <%s> set to "%s"', $attendee->Value(), $response );
+    $attendee->SetParameterValue( 'SCHEDULE-STATUS', $response );
+    $scheduling_actions = true;
   }
+
+  if ( !$create ) {
+    foreach( $removed_attendees AS $attendee ) {
+      $schedule_target = new Principal('email',$email);
+      if ( $schedule_target->Exists() ) {
+        $attendee_calendar = new WritableCollection(array('path' => $schedule_target->internal_url('schedule-default-calendar')));
+      }
+    }
+  }
+  return $scheduling_actions;
 }
 
 
@@ -566,7 +735,11 @@ EOSQL;
       if ( $qry->Exec('PUT',__LINE__,__FILE__) && $qry->rows() == 0 ) {
         $params[':olson_name'] = $olson;
         $params[':vtimezone'] = (isset($tz) ? $tz->Render() : null );
-        $qry->QDo('INSERT INTO timezones (tzid, olson_name, active, vtimezone) VALUES(:tzid,:olson_name,false,:vtimezone)', $params );
+        $params[':last_modified'] = (isset($tz) ? $tz->GetPValue('LAST-MODIFIED') : null );
+        if ( empty($params[':last_modified']) ) {
+          $params[':last_modified'] = gmdate('Ymd\THis\Z');
+        }
+        $qry->QDo('INSERT INTO timezones (tzid, olson_name, active, vtimezone, last_modified) VALUES(:tzid,:olson_name,false,:vtimezone,:last_modified)', $params );
       }
     }
     else {
@@ -725,13 +898,13 @@ function write_attendees( $dav_id, vCalendar $ical ) {
 * 
 * @return boolean True for success, false for failure.
 */
-function write_resource( DAVResource $resource, $caldav_data, DAVResource $collection, $author, $etag, $put_action_type, $caldav_context, $log_action=true, $weak_etag=null ) {
+function write_resource( DAVResource $resource, $caldav_data, DAVResource $collection, $author, &$etag, $put_action_type, $caldav_context, $log_action=true, $weak_etag=null ) {
   global $tz_regex, $session;
 
   $path = $resource->bound_from();
   $user_no = $collection->user_no();
-  $ic = new vCalendar( $caldav_data );
-  $resources = $ic->GetComponents('VTIMEZONE',false); // Not matching VTIMEZONE
+  $vcal = new vCalendar( $caldav_data );
+  $resources = $vcal->GetComponents('VTIMEZONE',false); // Not matching VTIMEZONE
   if ( !isset($resources[0]) ) {
     $resource_type = 'Unknown';
     /** @todo Handle writing non-calendar resources, like address book entries or random file data */
@@ -741,7 +914,7 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
   else {
     $first = $resources[0];
     if ( !($first  instanceof vComponent) ) {
-      print $ic->Render();
+      print $vcal->Render();
       fatal('This is not a vComponent!');
     }
     $resource_type = $first->GetType();
@@ -765,17 +938,19 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
   );
 
   if ( $put_action_type == 'INSERT' ) {
-    $qry->QDo('SELECT nextval(\'dav_id_seq\') AS dav_id');
+    $qry->QDo('SELECT nextval(\'dav_id_seq\') AS dav_id, null AS caldav_data');
   }
   else {
-    $qry->QDo('SELECT dav_id FROM caldav_data WHERE dav_name = :dav_name ', array(':dav_name' => $path));
+    $qry->QDo('SELECT dav_id, caldav_data FROM caldav_data WHERE dav_name = :dav_name ', array(':dav_name' => $path));
   }
   if ( $qry->rows() != 1 || !($row = $qry->Fetch()) ) {
     // No dav_id?  => We're toast!
+    trace_bug( 'No dav_id for "%s" on %s!!!', $path, ($create_resource ? 'create': 'update'));
     rollback_on_error( $caldav_context, $user_no, $path);
     return false;
   }
   $dav_id = $row->dav_id;
+  $old_dav_data = $row->caldav_data;
   $dav_params[':dav_id'] = $dav_id;
   $calitem_params[':dav_id'] = $dav_id;
 
@@ -861,7 +1036,7 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
   $last_olson = 'Turkmenikikamukau';  // I really hope this location doesn't exist!
   $tzid = GetTZID($first);
   if ( !empty($tzid) ) {
-    $timezones = $ic->GetComponents('VTIMEZONE');
+    $timezones = $vcal->GetComponents('VTIMEZONE');
     foreach( $timezones AS $k => $tz ) {
       if ( $tz->GetPValue('TZID') != $tzid ) {
         /**
@@ -911,9 +1086,15 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
   $calitem_params[':percent_complete'] = $first->GetPValue('PERCENT-COMPLETE');
   $calitem_params[':status'] = $first->GetPValue('STATUS');
 
+  if ( !$collection->IsSchedulingCollection() ) {
+    if ( do_scheduling_requests($vcal, ($put_action_type == 'INSERT'), $old_dav_data ) ) {
+      $dav_params[':dav_data'] = $vcal->Render(null, true);
+      $etag = null;
+    }
+  }
+
   if ( !isset($dav_params[':modified']) ) $dav_params[':modified'] = 'now';
   if ( $put_action_type == 'INSERT' ) {
-    if ( !$collection->IsSchedulingCollection() ) do_scheduling_requests($ic,true);
     $sql = 'INSERT INTO caldav_data ( dav_id, user_no, dav_name, dav_etag, caldav_data, caldav_type, logged_user, created, modified, collection_id, weak_etag )
             VALUES( :dav_id, :user_no, :dav_name, :etag, :dav_data, :caldav_type, :session_user, :created, :modified, :collection_id, :weak_etag )';
     $dav_params[':collection_id'] = $collection_id;
@@ -922,7 +1103,6 @@ function write_resource( DAVResource $resource, $caldav_data, DAVResource $colle
     $dav_params[':created'] = (isset($created) && $created != '' ? $created : $dtstamp);
   }
   else {
-    if ( !$collection->IsSchedulingCollection() ) do_scheduling_requests($ic,false);
     $sql = 'UPDATE caldav_data SET caldav_data=:dav_data, dav_etag=:etag, caldav_type=:caldav_type, logged_user=:session_user,
             modified=:modified, weak_etag=:weak_etag WHERE dav_id=:dav_id';
   }
@@ -964,7 +1144,7 @@ EOSQL;
   }
 
   write_alarms($dav_id, $first);
-  write_attendees($dav_id, $ic);
+  write_attendees($dav_id, $vcal);
 
   if ( $log_action && function_exists('log_caldav_action') ) {
     log_caldav_action( $put_action_type, $first->GetPValue('UID'), $user_no, $collection_id, $path );
